import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Widget/baseWidgets.dart';

class BaseWidgetBuilder<T extends BaseController> extends BaseWidget<T> {
  BaseWidgetBuilder({
    @required T controller,
    @required this.builder,
    Function(GetBuilderState<T> state) initState,
    Function(GetBuilderState<T> state) dispose,
    Function(GetBuilderState<T> state) didChangeDependencies,
    Function(GetBuilder oldWidget, GetBuilderState<T> state) didUpdateWidget,
    this.skeletonBuilder,
    this.toStringBuilder,
  })  : this._initState = initState,
        this._dispose = dispose,
        this._didChangeDependencies = didChangeDependencies,
        this._didUpdateWidget = didUpdateWidget,
        super(controller);

  final Widget Function(
    BuildContext context,
    Map<String, List<Map<String, Object>>> data,
    T controller,
  ) builder;
  final Function(GetBuilderState<T> state) _initState;
  final Function(GetBuilderState<T> state) _dispose;
  final Function(GetBuilderState<T> state) _didChangeDependencies;
  final Function(GetBuilder oldWidget, GetBuilderState<T> state)
      _didUpdateWidget;
  final Widget Function(BuildContext context, T controller) skeletonBuilder;
  final String Function(DiagnosticLevel minLevel) toStringBuilder;

  @override
  void initState(GetBuilderState<T> state) {
    super.initState(state);
    _initState?.call(state);
  }

  @override
  void dispose(GetBuilderState<T> state) {
    super.dispose(state);
    _dispose?.call(state);
  }

  @override
  void didUpdateWidget(GetBuilder oldWidget, GetBuilderState<T> state) {
    super.didUpdateWidget(oldWidget, state);
    _didUpdateWidget?.call(oldWidget, state);
  }

  @override
  void didChangeDependencies(GetBuilderState<T> state) {
    super.didChangeDependencies(state);
    _didChangeDependencies?.call(state);
  }

  @override
  String toString({DiagnosticLevel minLevel = DiagnosticLevel.info}) =>
      toStringBuilder?.call(minLevel) ?? super.toString(minLevel: minLevel);

  @override
  Widget getMainView(
    BuildContext context,
    Map<String, List<Map<String, Object>>> data,
    BaseController baseController,
  ) {
    return builder?.call(context, data, baseController) ??
        getSkeletonView(context, baseController);
  }

  @override
  Widget getSkeletonView(
    BuildContext context,
    BaseController baseController,
  ) {
    return skeletonBuilder?.call(context, baseController) ??
        super.getSkeletonView(context, baseController);
  }
}
