part of edit_screen;

extension EditorUpload on EditorController {
  Future<Map<String, Object>> save({
    VoidCallback onSuccess,
    bool useLoadingDialog = false,
    Function(
      bool success,
      Map<String, List<Map<String, Object>>> response,
    )
        afterSave,
    bool checkDirtyData = true,
    bool uploadImage = true,
    Map<String, Object> params,
  }) async {
    Map<String, Object> dataReturn = {};
    if (onSuccess != null) {
      this.onSuccess = onSuccess;
    }
    if (_isDirtyData || !checkDirtyData) {
      final Map<String, Object> jParam = dataEntity.map((key, value) {
        if (value is num || value is bool || value is String) {
          value = value;
        } else {
          value = value?.toString();
        }
        return MapEntry(key, value);
      });
      jParam.removeWhere(
        (key, value) => _checkIsImagesKey(key.replaceFirst(RegExp(r'@'), '')),
      );

      params ??= {};
      jParam.forEach((key, value) {
        params.putIfAbsent(key, () => value);
      });

      try {
        if (useLoadingDialog) _isWaiting.value = true;
        print('EditorWidget: Submitting...');

        final response =
            await SQLService.execute(onSaveProcedure, params: params);
        print('result: $response');
        dataReturn = response;
        if (response['Exception']?.isNotEmpty ?? false) {
          _succeed.value = false;
          afterSave?.call(false, response);
          print(
              'Editor Save Error On Server: ${response['Exception']?.first['Message']}');
        } else {
          if (uploadImage) {
            await saveImages(response);
          }
          _succeed.value = true;
          afterSave?.call(true, response);
          onSuccess?.call();
        }
      } catch (e, stackTrace) {
        print('Editor Save Error: $e');
        print(stackTrace);
      }
      EditorController.mapDataReturn = dataReturn;
      return dataReturn;
    } else {
      print('data have already save');
      EditorController.mapDataReturn = dataReturn;
      return dataReturn;
    }
  }

  Future<void> saveImages(
    Map<String, List<Map<String, Object>>> response,
  ) async {
    final imageEditorItems = editorItems
        .where((element) => element.type == EEditorType.image)
        .toList();
    if (imageEditorItems.isEmpty) return;
    final futures = <Future>[];
    for (final imageEditorItem in imageEditorItems) {
      var value = dataEntity[imageEditorItem.fieldName];
      if (value == null) continue;
      List<File> images = Methods.getListFile(value);
      UploadImageInfo uploadImageInfo = imageEditorItem.uploadImageInfo;
      for (var data in response[uploadImageInfo.responseIdsName]) {
        if (data == null) continue;
        // id của dữ liệu chính
        int id = Methods.getInt(data, 'ID');
        if (id > 0) {
          for (File image in images) {
            Future<String> upload;
            if (uploadImageInfo.parentTableName != null &&
                uploadImageInfo.parentTableName.isNotEmpty &&
                uploadImageInfo.parentTableName != uploadImageInfo.tableName) {
              upload = Methods.uploadFileParentChild(
                image,
                uploadImageInfo.tableName,
                -1,
                uploadImageInfo.fieldName,
                uploadImageInfo.parentTableName,
                id,
              );
            } else {
              upload = Methods.uploadFileParent(
                image,
                uploadImageInfo.tableName,
                id,
                uploadImageInfo.fieldName,
              );
            }
            futures.add(
              upload.then((value) {
                print("Result upload image: $value");
                if (response[uploadImageInfo.tableName] == null) {
                  response[uploadImageInfo.tableName] = <Map<String, Object>>[];
                }
                var list = response[uploadImageInfo.tableName];
                list.add({
                  'ID': Methods.getFileId(value),
                  'refID': id,
                  'hinhAnh_image_path': Methods.getFilePath(value),
                });
              }, onError: (err) {
                print('Error upload image: $err');
              }),
            );
          }
        }
      }
    }

    await Future.wait(futures);
  }
}
