import 'package:flutter/foundation.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wowit/model.dart';
import 'package:wowit/sqlService.dart';
import 'common.dart';
import 'methods.dart';

class PickerRange {
  PickerRange._();
  static const String picker = 'picker';
  static const String range = 'range';

  static const String splitTable = '[::]';
  static const String splitValue = '[..]';
  static const String splitData = '[,,]';

  static KeyValue getDefaultPicker({String tableName}) {
    if (kDebugMode) {
      if (tableName == null) tableName = "";
      return KeyValue(
          -1, -1, "Chưa nhập liệu cho drop dơn này: " + tableName.toString());
    } else {
      return KeyValue(-1, -1, "--");
    }
  }

  static init(
    Map<String, List<Map<String, Object>>> reponse, {
    bool reload = false,
  }) async {
    if (reload) {
      try {
        if (reponse == null || reponse.isEmpty) {
          reponse = await SQLService.execute('sp_InitSetting',
              params: {'@getPickerRange': true});
          if (reponse.length == 0) {
            return;
          }
        }
        reponse.forEach((key, value) {
          String tableName = key;
          print("Picker-Range init: " + tableName);
          if (tableName.startsWith('DataRange_')) {
            PickerRange.initDataRange(
                tableName.replaceAll('DataRange_', ''), value);
          } else if (tableName.startsWith('DataPicker_')) {
            PickerRange.initDataPicker(
                tableName.replaceAll('DataPicker_', ''), value);
          }
        });
        savePickerToLocal();
        saveRangeToLocal();
        SQLService.execute('sp_updatePicker');
      } catch (ex) {
        print("PickerRange init exception: " + ex.toString());
      }
    } else {
      loadPickerFromLocal();
      loadRangeFromLocal();
    }
  }

  static String getDataPicker(List<KeyValue> data, int id,
      {String defaultValue = ""}) {
    for (int i = 0; i < data.length; i++) {
      try {
        if (data[i].valueMember == id) {
          return data[i].displayMember;
        }
      } catch (ex) {
        print('getDataPicker ex:' + ex.toString());
      }
    }

    return defaultValue;
  }

  static List<KeyValue> getValueDataPicker(String tableName,
      {int parentId = -1}) {
    if (Common.mapPicker.containsKey(tableName)) {
      if (parentId == null || parentId == -1)
        return Common.mapPicker[tableName];
      List<KeyValue> data = Common.mapPicker[tableName];
      List<KeyValue> lst = [];
      for (int i = 0; i < data.length; i++) {
        if (data[i].parentValue == parentId) {
          lst.add(data[i]);
        }
      }
      if (lst.length == 0 && parentId == -1) {
        lst.add(PickerRange.getDefaultPicker());
        if (kDebugMode) {
          SQLService.execute('sp_AddPickerRange',
              params: {'@tableName': tableName});
        }
      }
      return lst;
    } else {
      if (kDebugMode) {
        SQLService.execute('sp_AddPickerRange',
            params: {'@tableName': tableName});
      }
    }

    return [PickerRange.getDefaultPicker()];
  }

  static Future<List<KeyValue>> initDataPicker(
      String tableName, List<Map<String, Object>> dtData,
      {bool override = false}) async {
    if (Common.mapPicker.containsKey(tableName) && !override)
      return Common.mapPicker[tableName];
    List<KeyValue> listVal = [];
    try {
      if (dtData == null || dtData.isEmpty) {
        var ds = await SQLService.execute('sp_getValueDataPicker',
            params: {'@tableName': tableName});
        if (ds.length == 0) return listVal;
        dtData = ds["sp_getValueDataPicker"];
      }

      for (var row in dtData) {
        int id = Methods.getInt(row, 'ID', defaultValue: -1);
        int parentId = Methods.getInt(row, 'parentId');
        String title = Methods.getString(row, 'title');
        if (id < 0) continue;
        listVal.add(KeyValue(parentId, id, title));
      }
      Common.mapPicker[tableName] = listVal;
    } catch (ex) {
      print('[Exception] getValueDataPicker: ' + ex.toString());
    }
    print('getValueDataPicker: ' +
        tableName +
        ' count: ' +
        listVal.length.toString());
    return listVal;
  }

  static Map<int, RangeValue> getValueDataRange(String tableName) {
    if (Common.mapRange.containsKey(tableName)) {
      return Common.mapRange[tableName];
    } else if (kDebugMode) {
      SQLService.execute('sp_AddPickerRange',
          params: {'@tableName': tableName});
    }
    return {};
  }

  static Future<Map<int, RangeValue>> initDataRange(
      String tableName, List<Map<String, Object>> dtData,
      {bool override = false}) async {
    if (Common.mapRange.containsKey(tableName) && !override)
      return Common.mapRange[tableName];
    Map<int, RangeValue> listVal = {};
    try {
      if (dtData == null || dtData.isEmpty) {
        var ds = await SQLService.execute('sp_getValueDataRange',
            params: {'@tableName': tableName});
        if (ds.length == 0) return listVal;
        dtData = ds["sp_getValueDataRange"];
      }

      for (var row in dtData) {
        int id = Methods.getInt(row, 'ID', defaultValue: -1);
        String title = Methods.getString(row, 'title');
        double from = Methods.getDouble(row, 'valFrom', defaultValue: 0);
        double to = Methods.getDouble(row, 'valTo', defaultValue: 0);
        if (id < 0) continue;
        listVal[id] = RangeValue(from, to, title: title);
      }
      Common.mapRange[tableName] = listVal;
    } catch (ex) {
      print('[Exception] initDataRange: ' + ex.toString());
    }
    print('getValueDataPicker: ' +
        tableName +
        ' count: ' +
        listVal.length.toString());
    return listVal;
  }

  static RangeValue getRangeValueDataPicker(
      Map<int, RangeValue> rangeMap, int id,
      {RangeValue defaultValue = const RangeValue(0, 0)}) {
    return rangeMap[id] ?? defaultValue;
  }

  static void savePickerToLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> pickers = Common.mapPicker.entries.map<String>((e) {
      return e.key +
          splitTable +
          e.value.map<String>((e) {
            String parentValue = e.parentValue?.toString() ?? '0';
            String valueMember = e.valueMember?.toString() ?? '0';
            String displayMember =
                e.displayMember.isNotEmpty ? e.displayMember : ' ';
            return parentValue +
                splitValue +
                valueMember +
                splitValue +
                displayMember;
          }).join(splitData);
    }).toList();

    prefs.setStringList(picker, pickers);
    if (kDebugMode) {
      print('save Picker to local success');
    }
  }

  static void saveRangeToLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> ranges = Common.mapRange.entries.map<String>((e) {
      return e.key +
          splitTable +
          e.value.entries.map<String>((e) {
            String id = e.key?.toString() ?? '0';
            String start = e.value?.start?.toString() ?? '0';
            String end = e.value?.start?.toString() ?? '0';
            String title = e.value?.title ?? ' ';
            title = title.isNotEmpty ? title : ' ';
            return id +
                splitValue +
                start +
                splitValue +
                end +
                splitValue +
                title;
          }).join(splitData);
    }).toList();

    prefs.setStringList(range, ranges);
    if (kDebugMode) {
      print('save Range to local success');
    }
  }

  static void loadPickerFromLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> pickers = prefs.getStringList(picker);
    Common.mapPicker = Map.fromIterable(
      pickers,
      key: (e) {
        String key = e as String;
        return key.split(splitTable).first;
      },
      value: (e) {
        String dataset = e as String;
        return dataset.split(splitTable).last.split(splitData).map((e) {
          List<String> splits = e.split(splitValue);
          int parentValue = Methods.toInt(splits[0]);
          int valueMember = Methods.toInt(splits[1]);
          String displayMember = splits[2];
          return KeyValue(parentValue, valueMember, displayMember);
        }).toList();
      },
    );
    if (kDebugMode) {
      print('//==============================');
      print('||load Picker from local success');
      Common.mapPicker.forEach((key, value) {
        print('||$key: ${value.length} item');
      });
      print('\\\\==============================');
    }
  }

  static void loadRangeFromLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> ranges = prefs.getStringList(range);
    Common.mapRange = Map.fromIterable(
      ranges,
      key: (e) {
        String key = e as String;
        return key.split(splitTable).first;
      },
      value: (e) {
        String dataset = e as String;
        return Map.fromIterable(
          dataset.split(splitTable).last.split(splitData),
          key: (e) {
            String key = e as String;
            return Methods.toInt(key.split(splitValue).first);
          },
          value: (e) {
            String rangeValue = e as String;
            List<String> splits = rangeValue.split(splitValue);
            double start = Methods.toDouble(splits[1]);
            double end = Methods.toDouble(splits[2]);
            String title = splits[3];
            return RangeValue(start, end, title: title);
          },
        );
      },
    );
    if (kDebugMode) {
      print('//===========================');
      print('load Range from local success');
      Common.mapRange.forEach((key, value) {
        print('||$key: ${value.length} item');
      });
      print('\\\\===========================');
    }
  }
}
